@@ -8,6 +8,7 @@ import time
 
 from django.db import DEFAULT_DB_ALIAS, DatabaseError
 from django.db.backends.mysql import base
+from django.utils.functional import cached_property
 
 LOGGER = logging.getLogger(__name__)
 
@@ -251,7 +252,17 @@ class CursorWrapper:
         raise exc
 
 
+class DatabaseFeatures(base.DatabaseFeatures):
+    @cached_property
+    def update_can_self_select(self):
+        if self.connection.disable_update_can_self_select:
+            return False
+        return super(DatabaseFeatures, self).update_can_self_select
+
+
 class DatabaseWrapper(base.DatabaseWrapper):
+    features_class = DatabaseFeatures
+
     base_settings = None
     failover_history = None
     failover_history_size = 0
@@ -270,9 +281,10 @@ class DatabaseWrapper(base.DatabaseWrapper):
         if 'OPTIONS' not in self.base_settings:
             self.base_settings['OPTIONS'] = dict()
         self.base_settings['OPTIONS'].pop('unix_socket', None)
+        self.disable_update_can_self_select = self.base_settings['OPTIONS'].pop('disable_update_can_self_select', True)
         self.failover_enable = self.base_settings['OPTIONS'].pop('failover_enable', True)
         self.failover_history = list()
-        self.failover_history_limit = self.base_settings['OPTIONS'].pop('failover_history_limit', 1000)
+        self.failover_history_limit = self.base_settings['OPTIONS'].pop('failover_history_limit', 10000)
         self.optimistic_transactions = self.base_settings['OPTIONS'].pop('optimistic_transactions', True)
         self.wsrep_sync_after_write = self.base_settings['OPTIONS'].pop('wsrep_sync_after_write', True)
         self.wsrep_sync_use_gtid = self.base_settings['OPTIONS'].pop('wsrep_sync_use_gtid', False)
@@ -421,14 +433,15 @@ class DatabaseWrapper(base.DatabaseWrapper):
                         self._wsrep_sync_wait_upto_gtid()
                     else:
                         self._wsrep_sync_wait(self.secondary_wrapper.connection)
-                    self.secondary_synced = True
                 except Exception as e:
                     error_code = str(e.args[0]) if e.args else ''
                     if retry < 3 and error_code == '1205':  # Lock wait timeout exceeded; try restarting transaction
                         LOGGER.info('Retry syncing secondary after: %s' % str(e), exc_info=True)
                         retry += 1
                         continue
-                    raise e
+                    LOGGER.warning('Error while syncing secondary: %s' % str(e), exc_info=True)
+                finally:
+                    self.secondary_synced = True
             t = time.perf_counter() - t
             LOGGER.debug('Secondary synced in %f seconds' % t)
 
@@ -456,10 +469,12 @@ class DatabaseWrapper(base.DatabaseWrapper):
     def _wsrep_sync_wait(self, connection):
         with connection.cursor() as cursor:
             cursor.execute(
+                'SET @lock_wait_timeout_orig = @@lock_wait_timeout;'
                 'SET @wsrep_sync_wait_orig = @@wsrep_sync_wait;'
                 'SET SESSION lock_wait_timeout = 5;'
-                'SET SESSION wsrep_sync_wait = GREATEST(@wsrep_sync_wait_orig, 1);'
+                'SET SESSION wsrep_sync_wait = 1;'
                 'SELECT 1;'
+                'SET SESSION lock_wait_timeout = @lock_wait_timeout_orig;'
                 'SET SESSION wsrep_sync_wait = @wsrep_sync_wait_orig;'
             )
 
